;                              
;                                  
;                            
;============================================================================
; v1.0 by Andy Selle --------------------------- (c)1996 All Rights Reserved
;============================================================================


#include "ti-85.h"

TempLine = $80df
PicCoor  = $80e7    
offset   = $80e9
NASR_Status = $80ea


randvar = $80EB
VARN = $80EC
VARS = $80ED
VARI = $80EE
VARB = $80EF
Count = $80F0
TempByte = $80F1
TempByte2 = $80F2
locx = $80F7
locy = $80F9
Flag = $80F3
Way = $80F5
vecx = $80FB
vecy = $80FD
gravity = $80FF
spmem = $8101
RES = $8103
DATA = $8A6B

.org 0
.db "Lunar Lander 1.0a",0

Start:
        ld hl,0
        add hl,sp
        ld (spmem),hl           ; Store the stack pointer away

        ld hl,randseed
        ld de,(PROGRAM_ADDR)
        add hl,de
        ld a,(hl)
        ld (randvar),a

        ld a,4
        out (5),a


        ROM_CALL(CLEARLCD)

	ld hl,(PROGRAM_ADDR)
	ld de,introstr
	add hl,de
	ROM_CALL(D_ZT_STR)

        ld a,(randvar)
        ld l,a
        ld h,0
        ROM_CALL(D_HL_DECI)
DrawRand:
        CALL GET_KEY
        cp K_MORE
        jr nz,DrawRand

        ROM_CALL(CLEARLCD)
        CALL_(DrawLand)
        ld hl,$0101
        ld ($8333),hl
        ld hl,(PROGRAM_ADDR)
        ld de,VectorXStr
        add hl,de
        ROM_CALL(D_ZM_STR)
        push hl
        ld hl,$0128
        ld ($8333),hl
        pop hl
        ROM_CALL(D_ZM_STR)
        push hl
        ld hl,$0150
        ld ($8333),hl
        pop hl
        ROM_CALL(D_ZM_STR)


        ld hl,$2000         ; Load starting module location
        ld (locx),hl
        ld (locy),hl
        ld hl,$0000
        ld (vecx),hl
        ld (vecy),hl
        ld hl,$0007
        ld (gravity),hl
        push hl
        ld hl,$015C
        ld ($8333),hl
        pop hl
        CALL_(ShowFix2)

InputLoop:
        HALT

Long_Delay:
        push af
        push bc
        ld a,$3F
        ld b,$3F
dloop:
        dec a
        jr nz,dloop
        dec b
        jr nz,dloop
        pop bc
        pop af
        CALL_(EraseLunar)

        ld a,0
        out (1),a

        in a,(1)
        cp 0
        jr z,InputLoop

        bit 6,a
        JUMP_Z(ExitProperly)
        bit 3,a
        jr nz,InputNextA
        push af
        ld hl,(vecy)
        ld de,$0025
        add hl,de
        ld (vecy),hl
        pop af
InputNextA:
        bit 0,a
        jr nz,InputNextB
        push af
        ld hl,(vecy)
        or a
        ld de,$0025
        sbc hl,de
        ld (vecy),hl
        pop af

InputNextB:

        bit 2,a
        jr nz,InputNext1
        push af
        ld hl,(vecx)
        ld de,$0010
        add hl,de

        ld (vecx),hl
        pop af
InputNext1:
        bit 1,a
        jr nz,InputNext2
        ld hl,(vecx)
        ld de,$0010
        or a
        sbc hl,de

        ld (vecx),hl

InputNext2:
        CALL_(UpdateLoc)
;        CALL_(CollisionDetect)
        CALL_(DrawLunar)
        ld hl,$010C
        ld ($8333),hl
        ld hl,(vecx)
        CALL_(ShowFix2)
        ld hl,$0134
        ld ($8333),hl

        ld hl,(vecy)
        CALL_(ShowFix2)

        JUMP_(InputLoop)
        ret

ExitProperly:
        ld sp,(spmem)
        CALL_(Rand)
        ld hl,randseed
        
        ld de,(PROGRAM_ADDR)
        add hl,de
        ld a,(randvar)

        ld (hl),a

   ld   hl,ZS_BITS
   SET  0,(hl)              ;Tell ZShell to warp out...
        ret


UpdateLoc:
        push af
        ld hl,(vecy)
        ld de,(gravity)
        or a
        sbc hl,de
        ld (vecy),hl

        ld hl,(locx)
        ld de,(vecx)
        add hl,de
        ld (locx),hl
        ld hl,(locy)
        ld de,(vecy)
        add hl,de
        ld (locy),hl
        pop af
        ret


CollisionDetect:
        ld hl,(locy)
        ld de,$0600
        or a
        sbc hl,de

        ld de,(locx)
        ld b,5
CollisLoop:
        push bc
        push af
        ld b,d
        ld c,h
        CALL_(GetPixel)
        cp 0
        JUMP_NZ(ExitProperly)
        inc b
        ld d,b
        pop af
        pop bc
        DJNZ CollisLoop


        ret
        


DrawLunar:
        ld hl,(locx)
        ld b,h
        ld hl,(locy)
        ld c,h
        ld a,1
        ld (NASR_Status),a
        ld hl,Module
        ld de,(PROGRAM_ADDR)
        add hl,de
        CALL_(NASRWARP)
        ret
EraseLunar:
        ld hl,(locx)
        ld b,h
        ld hl,(locy)
        ld c,h

        ld a,0
        ld (NASR_Status),a
        ld hl,Module
        ld de,(PROGRAM_ADDR)
        add hl,de
        CALL_(NASRWARP)
        ret

DrawLand:

        ld a,64
        ld (VARS),a
        ld a,1
        ld (VARN),a

        CALL_(Rand)
        ld a,(randvar)
        srl a
        srl a
        srl a
        srl a
        ld hl,DATA+1
        ld (hl),a
        
        CALL_(Rand)
        ld a,(randvar)
        srl a
        srl a
        srl a
        srl a
        ld hl,DATA+129
        ld (hl),a

        ld a,0
        ld (randvar),a

WhileLoop:
        ld a,(VARN)
        cp 128
        JUMP_Z(EndWhile)

        ld a,(VARS)
        inc a
        ld (VARB),a
        ld a,1
        ld (VARI),a
ForLoop:
        or a
        ld a,(VARS)
        ld b,a
        ld a,(VARB)
        sub b
        ld hl,DATA
        ld d,0
        ld e,a
        add hl,de
        ld a,(HL)         
        ld (TempByte),a
        ld a,(VARB)
        ld b,a
        ld a,(VARS)
        add a,b
        ld hl,DATA
        ld d,0
        ld e,a
        add hl,de
        ld a,(HL)

        ld b,a
        ld a,(TempByte)
        add a,b
        ld (TempByte),a
        CALL_(Rand)
        ld a,(randvar)
        ld e,a
        ld d,0
        ld a,(VARS)

        ; LALLA

        ld b,a
        CALL_(MULT)
        add hl,hl
        ld a,(VARS)
        srl a
        ld d,a
        ld e,0
        or a
        sbc hl,de
        ld a,(TempByte)
        ld d,a
        ld e,0
        add hl,de
        ld a,h
        srl a
        push af
        ld hl,DATA
        ld a,(VARB)
        ld e,a
        ld d,0
        add hl,de
        pop af

        cp 60
        jr c,MntOk
        ld a,60
MntOk:
        ld (hl),a

        push af
        ld a,(VARB)
        ld b,a
        pop af
        srl a
        srl a
        cp 0
        jr nc,CheckGoOn1
        ld a,0
CheckGoOn1:

        ld c,a
        CALL_(PlotPixel)


        ld a,(VARS)
        sla a
        ld b,a
        ld a,(VARB)
        add a,b
        ld (VARB),a



WaitY:
        ld a,(VARI)
        ld b,a
        ld a,(VARN)
        cp b
        jr z,EndFor
        inc b
        ld a,b
        ld (VARI),a
        JUMP_(ForLoop)
EndFor:
        ld a,(VARS)
        sra a
        ld (VARS),a

        ld a,(VARN)
        sla a
        ld (VARN),a
        JUMP_(WhileLoop)
EndWhile:
        ret



MULT:
        ld a,0
        ld hl,0
MULTLOOP:
        add hl,de
        DJNZ MULTLOOP

        ret
rand:
    ld    a,(randvar)  ;;must be defined in text area
    ld    b,a
    ld    a,0
    add   a,b
    sla   b
    sla   b
    add   a,b
    sla   b
    sla   b
    add   a,b
    inc   a
    ld    (randvar),a
    srl   a
    ret
;;end rand
PlotPixel:

        push bc
        ld a,b
        dec a
        ld b,a
        ROM_CALL(FIND_PIXEL)  ;HL=byte offset in video buffer, A=2^(bit to change)
        ld de, $FC00
        
        add hl, de
        push af
        or (hl)
        ld (hl),a

        ld b,c
        ld de,16
LineRout:
        add hl,de
        pop af
; USED TO AE        or (hl)
        or (hl)
        push af
        ld (hl),a
        DJNZ LineRout

        pop af
        pop bc
        ret

NASRWARP:    
        push    bc
        ex      de,hl
        ROM_CALL(FIND_PIXEL)
        ld      bc,$fc00
        add     hl,bc
        ld      (PicCoor),hl
        ex      de,hl
        ld      b,$ff
Patch:
        rla
        inc     b
        jr      nc,Patch
        ld      a,(hl)
        ld      c,a      
        inc     hl
        ld      a,(hl)
        ld      (offset),a
        ld      a,b   
        ld      e,c
        cp      (hl)
        jr      c,DS1
        inc     e
DS1:
        inc     hl
        ld      a,(hl)
        ld      d,a
        inc     hl
YLoop:            
        push    bc 
        push    de
        ld      b,8
        ex      de,hl
        ld      hl,TempLine
InitTempLineLoop:
        ld      (hl),$ff
        inc     hl
        djnz    InitTempLineLoop
        ex      de,hl
	ld	a,(NASR_Status)
	cp	0
	jr	z,NASRErase
NASRDraw:
        ld      b,0
        ld      de,TempLine
        ldir       
	jr	NASRDrawFin
NASRErase:
	ld	b,c
	ld	hl,TempLine
LoadEmpty:
	ld	(hl),0
	inc	hl
	djnz	LoadEmpty
NASRDrawFin:
        pop     de
        pop     bc
        push    hl
        push    bc         
        push    de
        ld      d,b
        ld      a,b
        cp      0
        jr      z,skipshift
SpriLoop1:
        ld      a,b    
        ld      HL,TempLine
        ld      b,e
        scf
SpriLoop2:        
        rr      (HL)
        inc     HL
        djnz    SpriLoop2
        ld      b,a
        djnz    SpriLoop1        
        ld      b,d
skipshift:
        ld      a,$ff           ;fill accumulator
        inc     b
mask1:
        srl     a               ;make the mask
        djnz    mask1           ;something like
        scf                     ;00001111 ->
        rl      a                        
        ld      hl,(PicCoor)    ;implement the mask
        or      (hl)            ;this preserve the 0 bits
        ld      hl,TempLine   ;become xxxx1111
        and     (hl)            ;when anded, become
        ld      (hl),a          ;xxxxyyyy
        ld      b,d             ;retrieve b
        ld      a,(offset)
        dec     a
        sub     b
        jr      nc,skip
        sub     248
skip:
        ld      b,a
        inc     b
        ld      a,$ff
mask2:               
        sla     a
        djnz    mask2
        scf
        rr      a
        dec     e
        ld      hl,(PicCoor)
        ld      d,0
        add     hl,de
        or      (hl)
        ld      hl,TempLine
        add     hl,de
        and     (hl)
        ld      (hl),a
        inc     e
        ld      c,e
        ld      hl,(PicCoor)
        ld      de,16
        push    hl
        add     hl,de
        ld      (PicCoor),hl
        pop     hl
        ld      de,TempLine
        ex      de,hl
        ld      b,0
        ldir      
        pop     de
        pop     bc
        pop     hl
        dec     d
        JUMP_NZ(YLoop)
        pop     bc
        ret


Line:                           ;beginning of the line routine
        push    DE
        push    HL

        ld      A,D
        sub     H
        jr      NC,DXpositive    ;delta_x > 0
DXNegative:
        neg
DXPositive:
        ld      B,A              ;B <- |delta_x|
        ld      A,E
        sub     L
        jr      NC,DYpositive    ;delta_y > 0
DYNegative:
        neg
DYPositive:                
        sub     B               ;|delta_y|
        jr      C,DeltaX        ;|delta_x| > |delta_y|
DeltaY:
        ld      A,H             ;if |delta_x| < |delta_y| then
        ld      H,L             ;then values x and y are swapped
        ld      L,A             ;so the loop will always be performed on the
        ld      A,D             ;x value. A flag must be set to 
        ld      D,E             ;remind that data must be drawn (y,x) 
        ld      E,A             ;instead of (x,y)

        ld      A,$01           
        ld      (Flag),A        ;the Flag !!!
DeltaX:
        ld      A,D
        sub     H
        jr      NC,TestDY        ;x1 < x2 
TestDX:
        ex	DE,HL
TestDY:
        ld      A,E
        sub     L
        ld      A,$01
        jr      NC,StoreA
        neg                     ;y1 > y2 : in case2 the 'y' variable
StoreA:
        ld      (Way),A
InitLine:       
        ld      B,H
        ld      C,L
        ld      A,E
        sub     L
        jr      NC,EndInit
        ld      A,L
        ld      L,E
        ld      E,A
EndInit:
        ld      A,E      
        sub     L
        rla
        ld      L,A             ;value to add in case1 (d < 0)
        add     A,H
        sub     D
        ld      E,A             ;'d' variable is initialised
        add     A,H
        sub     D
        ld      H,A             ;value to add in case2 (d >= 0)
Loop:
        ld      A,B
        sub     D
        jr      NC,EndLine       ;the line is completely drawn.
        ld      A,(Flag)
        or      A
        push    AF
        push    BC
        jr      Z,DrawPoint
        ld      A,B
        ld      B,C
        ld      C,A
DrawPoint:     
        CALL_(PointOn)
        pop     BC
        pop     AF
TestD:
        bit     7,E
        jr      NZ,Case1
Case2:                          ;d >= 0 
        ld      A,E
        add     A,H
        ld      E,A
        ld      A,(Way)
        add     A,C
        ld      C,A
        jr      EndLoop
Case1:                          ;d < 0
        ld      A,E
        add     A,L
        ld      E,A
EndLoop:
        inc     B
        jr      Loop
EndLine:
        pop     HL
        pop     DE
        ret                     ;end of the circle routine

PointOn:
        push    DE              ;MUST NOT BE REMOVED
        push    HL              ;MUST NOT BE REMOVED
        ROM_CALL(FIND_PIXEL)
        ld      DE,$FC00
        add     HL,DE
        xor      (HL)
        ld      (HL),A
        pop     HL              ;MUST NOT BE REMOVED
        pop     DE              ;MUST NOT BE REMOVED
        ret             

Wait:
        call    GET_KEY
        or      A
        jr      Z,Wait
        ret

GetPixel:               ; THANKS MAGNUS
        ROM_CALL(FIND_PIXEL)
        ld de,$FC00
        add hl,de
        and (hl)
        ret

ShowFix2:
        bit 7,h         ;Check sign
        jr z,notneg     ;Jump if positive
        ld de,$FFFF     ;16 bit not
        ex de,hl
        sbc hl,de
        inc hl          ;add 1
        ld a,'-'        ;Display sign
        jr actually
notneg:
        ld a,'+'
actually:
        ROM_CALL(M_CHARPUT)
        CALL_(ShowFix)  ;Show number
        ret


ShowFix:                
        push hl         ;Save number for later use
        ld a,h          ;Get integer part
        CALL_(ShowInt)  ;Show it
        ld a,'.'        ;Show decimal point
        ROM_CALL(M_CHARPUT)
        pop hl          ;Restore number
        ld a,l          ;Get fraction
        CALL_(ShowFrac) ;Show it
        ret

;------------------------------------------------------------------
;
; ShowInt
;
; Displays integer part of a fixpoint number (8 MSB)
; This rutines is based on the rutine used in TEXAN by
; Magnus Hagander (e95_mha@e.kth.se), but has been
; changed a bit by me :-)
;
; Entry : A holds value to be printed
; Exit  : A HL and DE destroyed
;
;------------------------------------------------------------------

ShowInt:
        ld l,a          ;Store value in hl
        ld a,0
        ld h,a
        ld de,RES+3     ;Location of the end of the string
        ld (de),a       ;Make string 0 terminated
        dec de          ;point to last digit
        ld b,3          ;Number of digits
ConvLoop:
        call UNPACK_HL  ;Unpack on digit from hl
        cp 0
        add a,'0'       ;make it ascii
        ld (de),a       ;store it
        dec de          ;point to next digit
        djnz ConvLoop
        ld hl,RES       ;point to string
        ROM_CALL(D_ZM_STR) ;Display it
        ret

;------------------------------------------------------------------
;
; ShowFrac
;
; Displays the fraction part of a fixpoint number on screen.
;
; Entry : A holds value (lower 8 bits of a number)
; Exit  : All regs destroyed
; Uses  : RES and the most of the rutines below
;------------------------------------------------------------------

ShowFrac:
        CALL_(Frac2bcd)         ;Convert number to BCD array
        ld hl,RES               ;Point to array
        CALL_(PrintArray)       ;Print number
        ret

;------------------------------------------------------------------
; Frac2bcd
;
; This procedure converts the fraction part of a fixpoint number to
; a 4 byte array containing the BCD values. Result is stored at adr.
; RES
;
; Entry : a hold the value to be converted
; Exit  : all regs destroyed
; Uses  : Addarray
;------------------------------------------------------------------

Frac2bcd:
        ld b,2                  ;changed from 8Number of bits in number
        ld hl,RES               ;point to string
        ld d,0                  ;Set result to 0
reset:
        ld (hl),d
        inc hl
        djnz reset
        ld hl,RES               ;point to string
        ld b,8                  ;number of bits
        push hl                 ;Store pointer to result
        ld hl,Cstart            ;Get adr. of first string
        ld de,(PROGRAM_ADDR)    ;Add program offset
        add hl,de
        ex de,hl                ;store in de
        pop hl                  ;Restore pointer to result
loopd:
        rr a                    ;Shift left
        jr nc,next              ;skip if bit=0
        push hl                 ;Save pointer to result
        CALL_(Addarray)         ;Add number
        pop hl                  ;Restore pointer to result
        jr next2                ;
next:
        dec de                  ;Point to next number
        dec de
        dec de
        dec de
next2:
        djnz loopd               ;Repeat for all bits
                                ;Now RES cointains a 8 byte bcd array with the 
result.
        ret
;------------------------------------------------------------------
;
; PrintBCD
;
; Prints the BCD number in A on the display. This function was
; originally called PrintHex and was made by Darryl Nester
; (email : nesterd@bluffton.edu)
;
; Entry : A holds value to be printed
; Exit  : No regs destroyed
;------------------------------------------------------------------

PrintBCD:
  push  af                      ;Save number
  rrca                          ;Make the 4 MSB's the 4 LSB's
  rrca
  rrca
  rrca
  CALL_(Digit)                  ;Display first digit
  pop   af                      ;Restore number
Digit:
  and $0F                       ;Use only lower 4 bits
  add a,'0'                     ;Make ascii
  ROM_CALL(M_CHARPUT)          ;Show digit
  ret

;------------------------------------------------------------------
;
; PrintArray
;
; Prints a 4 byte array of BCD numbers on the display.
;
; Entry : HL points to array
; Exit  : Destroys A and HL
; Uses  : uses PrintBCD to print each digit
;
;------------------------------------------------------------------

PrintArray:
        ld b,1                  ; changed from 4Number of bytes in array
PALoop:
        ld a,(hl)               ;Get byte
        CALL_(PrintBCD);        ;Print it
        inc hl                  ;point to next byte
        djnz PALoop
        ret

;------------------------------------------------------------------
; Addarray
;
; Addition of two 8 byte long bcd/numbers.
;
; Entry : hl points to start of array
;         de points to last byte in second array  
; Exit  : destroys bc,de,hl and flags
;------------------------------------------------------------------

Addarray:
        ld bc,$0003             ;Lenght of string-1
        add hl,bc               ;Point to end of string
        ld b,4                  ;Number of bytes
        ld c,a                  ;Save a reg
        and a                   ;clear carry
Addloop:
        ld a,(de)               ;Get first byte
        adc a,(hl)              ;Add second byte
        daa                     ;Make BCD
        ld (hl),a               ;Store result
        dec hl                  ;Point to next bytes
        dec de
        djnz Addloop            ;Do it for all bytes in array
        ld a,c                  ;Restore A reg
        ret                     ;Done

C7:
.db $50,$00,$00,$00             ;Arrays used for bin->ascii convertion of fracs.
C6:
.db $25,$00,$00,$00
C5:
.db $12,$50,$00,$00
C4:
.db $06,$25,$00,$00
C3:
.db $03,$12,$50,$00
C2:
.db $01,$56,$25,$00
C1:
.db $00,$78,$12,$50
C0:
.db $00,$39,$06
Cstart:                         ;Make it easy to point last byte
.db $25


Module: .db 1,4,5
        .db %00100000
        .db %01110000
        .db %01010000
        .db %11111000
        .db %10101000

;Module: .db 2,6,10
;            01234567   89
;       ---------------
;        .db %00000100, %00000000
;        .db %00001110, %00000000
;        .db %00001010, %00000000
;        .db %00001110, %00000000
;        .db %00011111, %00000000
;        .db %00011111, %00000000
;        .db %00101110, %10000000
;        .db %00100100, %10000000
;        .db %01000100, %01000000
;        .db %11001110, %01100000


randseed: .db %01100000
VectorXStr: .db "X: ",0
            .db "Y: ",0
            .db "G: ",0
IntroStr: .db "Hit more",0
.end



